C This program carries out nested multivariate analysis of variance
C by canonical redundancy analysis (db-RDA approach), using 2 or 3 factors
C (one main factor, plus 1 or 2 nested factors). If a single factor is
C present, the program computes simple Manova with permutation test.
C
C If a file for a single anova factor is provided, the program performs
C a one-way multivariate analysis of variance.
C
C Species abundance response data tables can be transformed by the program,
C using the transformations proposed by Legendre & Gallagher (2001).
 
C Y(n,p) among 1, 2, or 3 tables of explanatory variables.
C
C File 11 = Table Y of response variables. 
C           Begins with a header: n,p (n = # objects, p = # variables)
C
C File 12 = 1st table of explanatory variables. 
C           Begins with a header: n,m (n = # objects, m = # variables)
C
C File 13 = 2nd table of explanatory variables. Begins with a header: n,m
C
C File 14 = 3rd table of explanatory variables. Begins with a header: n,m
C
C File 2 = Number of files (Y and X) to be analysed, followed by the file names
C
C File 3 = Fitted fractions
C
C File 4 = "NesAnova.out" for nested anova results
C
C                              -----------------
C
C Basic references on db-RDA and the species data transformations:
C
C Legendre, P. & M. J. Anderson. 1999. Distance-based redundancy analysis: 
C    testing multispecies responses in multifactorial ecological experiments. 
C    Ecological Monographs 69 (1): 1-24.
C
C Legendre, P. and E. Gallagher. 2001. Ecologically meaningful transformations 
C    for ordination of species data. Oecologia 129: 271-280.
C
C Legendre, P., and L. Legendre. 1998. Numerical ecology, 2nd English 
C    edition. Elsevier, Amsterdam, The Netherlands.
C
C The relationship between the Object scores produced by this program
C and the "Sample scores that are linear compinations of the environmental 
C variables" (SS/LCEV) produced by Canoco is described in Legendre and
C Legendre (1998: 585).
C     
C                                              Pierre Legendre, May 2002
C
C Begun during Air New Zealand flight 15, Los Angeles to Sydney, 13 May 2002
C ----------------------------------------------------------------------
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
C Parameters determining the maximum size of the data tables:
C    nmax = maximum number of objects (rows of all data files)
C    pmax = maximum number of response variables in matrix Y
C    mmax = maximum n. of dummy var. in all files of anova factors combined
C    maxfileX = maximum number of files of anova factors
      Integer n,nmax,p,pmax,mtot,mmax,ntrmax
C     Parameter (nmax=501,pmax=500,mmax=200,maxfileX=3,
C    +           ntrmax=max(pmax,mmax))
C GNU Fortran g77 compiler: Replace the previous statement by
      Parameter (nmax=501,pmax=500,mmax=200,maxfileX=3,
     +           ntrmax=pmax)
C If 'pmax' or 'mmax' is modified, make sure that ntrmax=max(pmax,mmax)
      Integer m1(4),m2(4),iordre(nmax),nombre(ntrmax)
      Integer iGauss1(mmax),iGauss2(mmax),iGauss3(mmax)
      Real*8 GaussB(mmax,1)
      Real*8 Y(nmax,pmax),Yfitwk(nmax,pmax),
     +       Yfit1(nmax,pmax),Yfit2(nmax,pmax),Yfit3(nmax,pmax),
     +       Yfit7(nmax,pmax),
     +       Yres1(nmax,pmax),Yres2(nmax,pmax),Yres3(nmax,pmax),
     +       Yres4(nmax,pmax),Yres5(nmax,pmax),Yres6(nmax,pmax),
     +       Yres7(nmax,pmax),Yper(nmax,pmax),Yfitper(nmax,pmax)
C     La matrice X contient toutes les variables explicatives, provenant
C     de toutes les matrices explicatives individuelles
      Real*8 X(nmax,mmax),Xwk(nmax,mmax)
      Real*8 colsum(pmax),SSY
      Real*8 Trans(ntrmax,nmax),Inv(mmax,mmax),Work(nmax,mmax)
      Real*8 Resing1(nmax,nmax),Resing2(nmax,nmax),Resing3(nmax,nmax),
     +       Resing4(nmax,nmax),Resing5(nmax,nmax),Resing6(nmax,nmax),
     +       Resing7(nmax,nmax)
      Real*8 Z(nmax,ntrmax),U(ntrmax,ntrmax),vect(pmax),
     +       Lambda(ntrmax),EE(ntrmax)
      Character*79 nomfich
C
C BBEdit is the 'creator' for output files
C      call F_Creator('R*ch')
      idebug=0
      open(4,file='NesAnova.out',status='NEW')
      write(*,108)
      write(4,108)
      write(*,110) nmax,pmax,mmax,maxfileX
C
C File containing the names of the groups of files to be analysed
      write(*,*)
      write(*,*) 'Name of the file containing the groups of file names?'
      read(*,*) nomfich
      open(2,file=nomfich,status='OLD')
      inquire(unit=2,name=nomfich)
      write(*,109) nomfich
      write(4,109) nomfich
      read(2,*) nfiles
      if(nfiles.gt.4) Stop 'Too many files for this program.'
C Requested calculations
    2 write(*,*) 'Tasks:'
      write(*,*) '(0) Stop the program'
      write(*,*) '(1) First, compute the nested anova'
      write(*,*) '(2) Compute directly the canonical object scores'
      write(*,*) '(3) Both 1 and 2'
      read(*,*) job
      if(job.eq.0) Stop 'Program interrupted by user.'
      if((job.lt.1).or.(job.gt.3)) goto 2
      if(job.ge.2) then
         open(3,file='FittedSc.out',status='NEW')
         LUT=3
         write(LUT,*) 'Object scores fitted to the anova factors'
         write(LUT,*) 'Note: The response variables were centred ',
     +                'at the start of the calculations'
         write(LUT,*)
      write(LUT,*) '                          -------------------------'
         write(LUT,*)
         endif
C Transformation of species data?
      write(*,*)
    3 write(*,*) 'Transform species abundance data'
      write(*,*) '(Legendre & Gallagher, 2001)'
      write(*,*)
      write(*,*) '(0) No transformation (Euclidean distance preserved)'
      write(*,*) '(1) Chord distance'
      write(*,*) '(2) Chi-square metric'
      write(*,*) '(3) Chi-square distance'
      write(*,*) '(4) Distance between species profiles'
      write(*,*) '(5) Hellinger distance'
      read(*,*) ntran
      if((ntran.lt.0).or.(ntran.gt.5)) goto 3
C Permutation test?
C      write(*,*)
C    4 write(*,*) 'Permutation test:'
C      write(*,*) '(0) No permutation test'
C      write(*,*) '(1) Permute the raw data'
C      write(*,*) 
C     +   '(2) Permute residuals of the reduced model (recommended)'
C      read(*,*) mode
C      if((mode.lt.0).or.(mode.gt.2)) goto 4
C      if((mode.eq.0).or.(job.eq.2)) then
      if(job.eq.2) then
         nperm=0
         mode=0
         else
         mode=1
         write(*,*)
    5    write(*,*) 'Permutation test:'
         write(*,*) 'How many permutations? (e.g. 999, 9999, ...)'
         write(*,*) 'No permutation test: type 0'
         read(*,*) nperm
         if(nperm.lt.0) goto 5
         endif
C Check the coding of dummy variables?
      write(*,*)
    6 write(*,*) 
     +   'Check the coding of factors into orthogonal dummy variables?'
      write(*,*) '(0) No'
      write(*,*) '(1) Yes'
      read(*,*) icheck
      if((icheck.lt.0).or.(icheck.gt.1)) goto 6
C Start the clock
      debut=secnds(0.0)
C
C Read the data files
      call Read_Y(n,p,nmax,pmax,Y,colsum,ntran,SSY,idebug)
      mtot=0
      do i=1,4
         m1(i)=0
         m2(i)=0
         enddo
      iflag1a=0
      iflag1b=0
      iflag1c=0
      do ifilex=1,nfiles-1
         LIN=ifilex+11
         if(LIN.eq.12) then
            call Read_X(n,nmax,mmax,ntrmax,LIN,X,Xwk,Trans,Inv,m1,m2,
     +         nfiles,maxfileX,mtot,1,iflag1a,iflag2a,icheck,
     +         iGauss1,iGauss2,iGauss3,GaussB)
C      write(*,*) 'Point 3',iflag1,iflag2,iflag
            endif
         if(LIN.eq.13) then
            call Read_X(n,nmax,mmax,ntrmax,LIN,X,Xwk,Trans,Inv,m1,m2,
     +         nfiles,maxfileX,mtot,2,iflag1b,iflag2b,icheck,
     +         iGauss1,iGauss2,iGauss3,GaussB)
C      write(*,*) 'Point 4',iflag1,iflag2,iflag
            endif
         if(LIN.eq.14) then
            call Read_X(n,nmax,mmax,ntrmax,LIN,X,Xwk,Trans,Inv,m1,m2,
     +         nfiles,maxfileX,mtot,3,iflag1c,iflag2c,icheck,
     +         iGauss1,iGauss2,iGauss3,GaussB)
C      write(*,*) 'Point 5'
            endif
         enddo
C Report results of tests of orthogonality
      if(icheck.eq.1) then
         if(iflag1a+iflag1b+iflag1c.eq.0) then
            write(*,*)
            write(*,102)
            write(4,102)
           else
            write(*,*)
            write(*,103) 
            write(4,103) 
            stop
           endif
         endif
C Report results of tests of collinearity
      if(iflag2a+iflag2b+iflag2c.eq.0) then
         write(*,104) 
         write(4,104)
         endif
      if(mtot.ge.n) Stop 'Too many explanatory variables (mtot >= n)'
C Test collinearity among factors
      iflag=0
      if(nfiles.ge.3) then
         call CheckCol(n,nmax,mmax,ntrmax,maxfileX,X,Xwk,Trans,Inv,
     +      m1,m2,1,2,iflag1,iGauss1,iGauss2,iGauss3,GaussB)
         if(iflag1.ge.1) iflag=1
         endif
      if(nfiles.ge.4) then
         call CheckCol(n,nmax,mmax,ntrmax,maxfileX,X,Xwk,Trans,Inv,
     +      m1,m2,1,3,iflag1,iGauss1,iGauss2,iGauss3,GaussB)
         if(iflag+iflag1.ge.1) iflag=1
         call CheckCol(n,nmax,mmax,ntrmax,maxfileX,X,Xwk,Trans,Inv,
     +      m1,m2,2,3,iflag1,iGauss1,iGauss2,iGauss3,GaussB)
         if(iflag+iflag1.ge.1) iflag=1
         endif
      if(iflag.eq.1) then
         stop
         else
         write(*,105)
         write(4,105)
         endif
C Check the order and coding of the files of dummy variables
      ipos=0
      do i=1,nfiles-2
         inum1=m2(i)-ipos
         inum2=m2(i+1)-m2(i)
         ipos=m2(i)
         if(inum2.le.inum1) then
            write(*,101)
            write(4,101)
            stop
            endif
         enddo
C Centre the variables in X
      call Centre(n,nmax,1,mtot,mmax,X)
C Write the type and number of permutations to the output file
      write(4,*)
      write(4,106) nperm
C      if(mode.eq.1) write(4,106) nperm
C      if(mode.eq.2) write(4,107) nperm
C Carry out the (nested) anova
      if(nfiles.eq.2)
     +   call Part1(n,nmax,p,pmax,m1,m2,maxfileX,mtot,mmax,ntrmax,Y,X,
     +   Xwk,job,mode,nperm,Trans,Inv,Work,Resing1,
     +   iGauss1,iGauss2,iGauss3,GaussB,SSY,Yfit1,Yres1,Yper,Yfitper,
     +   Z,U,vect,Lambda,EE,iordre,nombre,LUT,debut)
      if(nfiles.eq.3) 
     +   call Part2(n,nmax,p,pmax,m1,m2,maxfileX,mtot,mmax,ntrmax,
     +   Y,X,Xwk,job,mode,nperm,Trans,Inv,Work,SSY,Yfitwk,
     +   Yfit1,Yfit2,Yfit3,Yres1,Yres2,Yres3,Yper,Yfitper,Z,U,vect,
     +   Resing1,Resing2,Resing3,iGauss1,iGauss2,iGauss3,GaussB,
     +   Lambda,EE,iordre,nombre,LUT,debut)
      if(nfiles.eq.4) 
     +   call Part3(n,nmax,p,pmax,m1,m2,maxfileX,mtot,mmax,ntrmax,
     +   Y,X,Xwk,job,mode,nperm,Trans,Inv,Work,SSY,
     +   Yper,Yfitper,Z,U,vect,Yfitwk,Yfit1,Yfit2,Yfit3,Yfit7,
     +   Yres1,Yres2,Yres3,Yres4,Yres5,Yres6,Yres7,Resing1,Resing2,
     +   Resing3,Resing4,Resing5,Resing6,Resing7,iGauss1,iGauss2,
     +   iGauss3,GaussB,Lambda,EE,iordre,nombre,LUT,debut)
C
      write(*,*) "Results are found in file 'NesAnova.out'"
      if(job.gt.1) write(*,*) "and in file 'FittedSc.out'"
      write(*,*)
      write(*,*) 'End of the program.'
      close(2)
      close(3)
      close(4)
      stop
  100 format(10f10.5)
  101 format(/' *** The input files are listed in an incorrect order'/
     + ' *** or the factors are not coded correctly. Program stopped.')
  102 format(' No problem was detected',
     +       ' in the orthogonal coding of the dummy variables')
  103 format(' *** Factor incorrectly coded into orthogonal dummy'/
     +       '     variables in one or more files')
  104 format(/' No collinearity detected within factor files')
  105 format(' No collinearity detected among factor files')
  106 format(' Probabilities after',i6,' permutations')
C  106 format(' Probabilities after permutation of raw data',
C     +       ' (',i6,' perm.)')
  107 format(' Probabilities after permutation of residuals',
     +       ' (',i6,' permutations)')
  108 format(' (Nested) multivariate analysis of variance'/
     +       ' by canonical redundancy analysis,'/ 
     +       ' using 1, 2, or 3 factors (one main factor,'/
     +       ' plus 0, 1, or 2 nested factors).'//
     +       ' Pierre Legendre'/
     +       ' Departement de sciences biologiques'/
     +       ' Universite de Montreal'/
     +       '             (c) Pierre Legendre, 2002'/)
  109 format(/' File containing the file names:   ',a32/)
  110 format(/' This program can handle a maximum of'/
     +        i7,' objects (rows of all data files)'/
     +        i7,' response variables in matrix Y'/
     +        i7,' dummy var. in all files of anova factors combined'/
     +        i7,' files of anova factors'/)
      end
      
      Subroutine Centre(n,nmax,j1,j2,mmax,Mat)
      Real*8 Mat(nmax,mmax),sx,xbar,dfln
C Centres columns j1 to j2 of matrix 'Mat' (nxm)
      dfln=dfloat(n)
      do 20 j=j1,j2
      sx=0.0
      do i=1,n
         sx=sx+Mat(i,j)
         enddo
      xbar=sx/dfln
      do i=1,n
         Mat(i,j)=Mat(i,j)-xbar
         enddo
   20 continue
      return
      end

      Subroutine CheckCol(n,nmax,mmax,ntrmax,maxfileX,X,Xwk,Trans,Inv,
     +      m1,m2,mat1,mat2,iflag1,
     +      iGauss1,iGauss2,iGauss3,GaussB)
      Integer m1(maxfileX),m2(maxfileX)
      Integer iGauss1(mmax),iGauss2(mmax),iGauss3(mmax)
      Real*8 GaussB(mmax,1)
      Real*8 X(nmax,mmax),Xwk(nmax,mmax),Trans(ntrmax,nmax),
     +       Inv(mmax,mmax),temp
C Check collinearity among files coding for factors
      iflag1=0
      mwk=0
      call SelectX(n,nmax,m1,m2,maxfileX,mat1,X,Xwk,mmax,mwk)
      call SelectX(n,nmax,m1,m2,maxfileX,mat2,X,Xwk,mmax,mwk)
      call Centre(n,nmax,1,mwk,mmax,Xwk)
      do i=1,n
         do j=1,mwk
            Trans(j,i)=Xwk(i,j)
            enddo
         enddo
      call Produit(Trans,Xwk,Inv, mwk,n,mwk, ntrmax,nmax, 
     +             nmax,mmax, mmax,mmax)
      call GaussJ(mwk,mmax,Inv,iGauss1,iGauss2,iGauss3,GaussB,iflag1)
      if(iflag1.eq.1) then
         write(*,102) mat1,mat2
         write(4,102) mat1,mat2
         endif
      return
  102 format
     + (' *** Collinearity detected between files X',i1,' and X',i1)
      end

      Subroutine Part1(n,nmax,p,pmax,m1,m2,maxfileX,mtot,mmax,ntrmax,
     +   Y,X,Xwk,job,mode,nperm,Trans,Inv,Work,Resing1,
     +   iGauss1,iGauss2,iGauss3,GaussB,SSY,Yfit1,Yres1,Yper,Yfitper,
     +   Z,U,vect,Lambda,EE,iordre,nombre,LUT,debut)
      Integer n,nmax,p,pmax,mmax,ntrmax
      Integer m1(maxfileX),m2(maxfileX),iordre(nmax),nombre(ntrmax)
      Integer iGauss1(mmax),iGauss2(mmax),iGauss3(mmax)
      Real*8 GaussB(mmax,1)
      Real*8 Y(nmax,pmax),Yfit1(nmax,pmax),Yres1(nmax,pmax),
     +       Yper(nmax,pmax),Yfitper(nmax,pmax)
      Real*8 X(nmax,mmax),Xwk(nmax,mmax)
      Real*8 Trans(ntrmax,nmax),Inv(mmax,mmax),
     +       Work(nmax,mmax),Resing1(nmax,nmax)
      Real*8 Z(nmax,ntrmax),U(ntrmax,ntrmax),vect(pmax),
     +       Lambda(ntrmax),EE(ntrmax)
      Real*8 SSY,A,B,F,Aper,Bper,Fper,Pperm,temp
      Character sig*1,fraction*9
C Part1: There is a single factor in the analysis
      write(4,*)
      write(4,120)
      write(4,101)
      write(4,120)
C Degrees of freedom for the F and Fperm statistics
      notest=0
      m=m2(1)
      idf1=m
      idf2=n-1-m
C Compute residualizing matrix  Residize(nxn) = X inv[X'X] X'
      call Residize(n,m, nmax,mmax, ntrmax,nmax, mmax,
     +       nmax,mmax, nmax, X,Trans,Inv,Work,Resing1,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      call Produit(Resing1,Y,Yfit1, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      do i=1,n
         do j=1,p
            Yres1(i,j)=Y(i,j)-Yfit1(i,j)
            enddo
         enddo
C Fractions: A = fitted, B = residuals
      call SofSq(n,p,nmax,pmax,Yfit1,A)
      B=SSY-A
C
      if((idf1.le.0).or.(idf2.le.0)) then
         notest=1
         goto 80
         endif
      F=A*dfloat(idf2)/(B*dfloat(idf1))
C Prepare for the permutation test
      if(nperm.eq.0) goto 80
      do i=1,n
         iordre(i)=i
         enddo
      iseed=0
      do i=1,100
         turn=rand(iseed)
         enddo
      iGEF=1
C Begin permutation test =====================================
      write(*,110)
      nval=n*p
      nlimit=1000
      do 90 iperm=1,nperm
      if((nval.gt.nlimit).and.(mod(iperm,10).eq.0)) write(*,130) iperm
  130 format('+','Permutation no',i6)
      call Permute(iseed,n,nmax,iordre)
C Permute raw data Y ***
C      if(mode.eq.1) then
         do 40 i=1,n
         do 40 j=1,p
   40    Yper(i,j)=Y(iordre(i),j)
C Permute Yres1 = [b], the residuals of the reduced regression model
C using the method described by Anderson & Legendre (1999, p. 281)
C         else
C         do 42 i=1,n
C         do 42 j=1,p
C   42    Yper(i,j)=Yres1(iordre(i),j)
C         endif
C
C     Compute A under permutation
      call Produit(Resing1,Yper,Yfitper, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      call SofSq(n,p,nmax,pmax,Yfitper,Aper)
      Bper=SSY-Aper
C     Compute Fper = (A/idf1) / (B/idf2)
      Fper=Aper*dfloat(idf2)/(Bper*dfloat(idf1))
      if(Fper.ge.F) iGEF=iGEF+1
C
   90 continue
C End of permutation test ===================================
C
      Pperm=dfloat(iGEF)/dfloat(nperm+1)
C Write results to output file
   80 continue
C Results for fraction A
      if(notest.eq.0) then
         if(mode.eq.0) then
            write(4,102) idf1,idf2,A,A/dfloat(idf1),F
            else
            sig=' '
            if(Pperm.le.0.05) sig='*'
            write(4,102) idf1,idf2,A,A/dfloat(idf1),F,Pperm,sig
            endif
         else
         write(4,102) idf1,idf2,A,A/dfloat(idf1)
         endif
C
      write(4,104) idf2,B,B/dfloat(idf2)
      write(4,120)
      write(4,105) (n-1),SSY
      write(4,120)
C
      fin=secnds(debut)
      write(*,111) fin
      write(4,111) fin
C Compute the fitted canonical scores
      if(job.eq.1) return
      debut=secnds(0.0)
      write(LUT,*) 'Object scores fitted to the anova factor'
      call PCA(n,nmax,p,pmax,nombre,Yfit1,Trans,ntrmax,U,Z,vect,Lambda,
     +         EE,SSY,LUT,1,1,nvalpr)
      write(LUT,*) '                          -------------------------'
      write(LUT,*)
      write(LUT,*) 
     +   'Object scores, PCA of unexplained variation (residuals)'
      call PCA(n,nmax,p,pmax,nombre,Yres1,Trans,ntrmax,U,Z,vect,Lambda,
     +         EE,SSY,LUT,0,1,nvalpr)
C
      fin=secnds(debut)
      write(*,112) fin
      write(LUT,112) fin
C
      return
C              123456789012345678901234512345123456789012345123456789012
  101 format( ' Source of variation  df1  df2               SS        ',
     +        '   MS           F      P(perm)')
  102 format( ' Anova factor       ',2i5,f20.5,f13.5,f12.5,f10.5,1x,a1)
  104 format(/' Within groups      ',i5,5x,f20.5,f13.5)
  105 format( ' Total              ',i5,5x,f20.5)
  110 format(/' Permutation test: please wait...'/)
  111 format(/' Computation time for anova:',f10.2,' sec.'/)
  112 format(/' Computation time for canonical object scores:',
     +       f10.2,' sec.'/)
  120 format(' -------------------------------------------------------',
     +        '--------------------------------')
      end
      
      Subroutine Part2(n,nmax,p,pmax,m1,m2,maxfileX,mtot,mmax,ntrmax,
     +   Y,X,Xwk,job,mode,
     +   nperm,Trans,Inv,Work,SSY,Yfitwk,Yfit1,Yfit2,Yfit3,
     +   Yres1,Yres2,Yres3,Yper,Yfitper,Z,U,vect,
     +   Resing1,Resing2,Resing3,iGauss1,iGauss2,iGauss3,GaussB,
     +   Lambda,EE,iordre,nombre,LUT,debut)
      Integer n,nmax,p,pmax,mmax,ntrmax
      Integer m1(maxfileX),m2(maxfileX),iordre(nmax),nombre(ntrmax)
      Integer iGauss1(mmax),iGauss2(mmax),iGauss3(mmax)
      Real*8 GaussB(mmax,1)
      Real*8 Y(nmax,pmax),Yfitwk(nmax,pmax),
     +       Yfit1(nmax,pmax),Yfit2(nmax,pmax),Yfit3(nmax,pmax),
     +       Yres1(nmax,pmax),Yres2(nmax,pmax),Yres3(nmax,pmax),
     +       Yper(nmax,pmax),Yfitper(nmax,pmax)
      Real*8 X(nmax,mmax),Xwk(nmax,mmax)
      Real*8 Trans(ntrmax,nmax),Inv(mmax,mmax),Work(nmax,mmax)
      Real*8 Resing1(nmax,nmax),Resing2(nmax,nmax),Resing3(nmax,nmax)
      Real*8 Z(nmax,ntrmax),U(ntrmax,ntrmax),vect(pmax),
     +       Lambda(ntrmax),EE(ntrmax)
      Real*8 SSY,AB,BC,ABC,B,D,
     +       ABper,BCper,ABCper,Dper
      Real*8 F1,F2,F1per,F2per,Pperm,temp
      Real*8 SSYper
      Character sig*1,fraction*9
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
      write(4,*)
      write(4,120)
      write(4,101)
      write(4,120)
C Part2: There are 2 factors: main A, nested B(A)
C
C 1. Compute [a] = [a+b] -----
C Degrees of freedom for the F1 and F1per statistics
      notest1=0
      idf1_1=m2(1)
      idf2_1=m2(2)-m2(1)
      mwk=0
      call SelectX(n,nmax,m1,m2,maxfileX,1,X,Xwk,mmax,mwk)
C Compute residualizing matrix  Resing1(nxn) = X inv[X'X] X'
      call Residize(n,mwk, nmax,mmax, ntrmax,nmax, mmax,
     +       nmax,mmax, nmax, Xwk,Trans,Inv,Work,Resing1,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      call Produit(Resing1,Y,Yfit1, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      do i=1,n
         do j=1,p
            Yres1(i,j)=Y(i,j)-Yfit1(i,j)
            enddo
         enddo
C Yres1 contains the residuals of Y after fitting factor A
C Fractions: AB = fitted, CD = residuals
      call SofSq(n,p,nmax,pmax,Yfit1,AB)
C
C 2. Compute [b] = [b+c] -----
C Degrees of freedom for the F2 and F2per statistics
      notest2=0
      idf1_2=m2(2)-m2(1)
      idf2_2=n-1-m2(2)
      mwk=0
      call SelectX(n,nmax,m1,m2,maxfileX,2,X,Xwk,mmax,mwk)
C Compute residualizing matrix  Resing2(nxn) = X inv[X'X] X'
      call Residize(n,mwk, nmax,mmax, ntrmax,nmax, mmax,
     +       nmax,mmax, nmax, Xwk,Trans,Inv,Work,Resing2,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      call Produit(Resing2,Y,Yfit2, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      do i=1,n
         do j=1,p
            Yres2(i,j)=Y(i,j)-Yfit2(i,j)
            enddo
         enddo
C Yres2 contains the residuals of Y after fitting factor B
C Fractions: BC = fitted, AD = residuals
      call SofSq(n,p,nmax,pmax,Yfit2,BC)
C
C 3. Compute [a+b+c] and [d] -----
C Degrees of freedom for the F3 and F3per statistics
      notest3=0
      mwk=0
      call SelectX(n,nmax,m1,m2,maxfileX,1,X,Xwk,mmax,mwk)
      call SelectX(n,nmax,m1,m2,maxfileX,2,X,Xwk,mmax,mwk)
C Compute residualizing matrix  Resing3(nxn) = X inv[X'X] X'
      call Residize(n,mwk, nmax,mmax, ntrmax,nmax, mmax,
     +       nmax,mmax, nmax, Xwk,Trans,Inv,Work,Resing3,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      call Produit(Resing3,Y,Yfit3, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      do i=1,n
         do j=1,p
            Yres3(i,j)=Y(i,j)-Yfit3(i,j)
            enddo
         enddo
C Fractions: ABC = fitted, D = residuals
      call SofSq(n,p,nmax,pmax,Yfit3,ABC)
      D=SSY-ABC
C
C Test statistics for nested anova
      if((idf1_1.le.0).or.(idf2_1.le.0).or.(BC.le.0.0)) notest1=1
      if((idf1_2.le.0).or.(idf2_2.le.0).or.(D.le.0.0)) notest2=1
      F1=AB*dfloat(idf2_1)/(BC*dfloat(idf1_1))
      F2=BC*dfloat(idf2_2)/(D*dfloat(idf1_2))
C 6. Compute [b] to check the orthogonality of the coded factors
      B=AB+BC-ABC
      if(B.gt.0.000000001) then
         write(*,*) 'Error in coding the nested factors: [b] =',B
         Stop
         endif
C
C Prepare for the permutation tests
      if(nperm.eq.0) goto 80
      do i=1,n
         iordre(i)=i
         enddo
      iseed=0
      do i=1,100
         turn=rand(iseed)
         enddo
      iGEF1=1
      iGEF2=1
C Begin permutation test =====================================
      write(*,110)
      nval=n*p
      nlimit=500
      do 90 iperm=1,nperm
      if((nval.gt.nlimit).and.(mod(iperm,10).eq.0)) write(*,130) iperm
  130 format('+','Permutation no',i6)
      call Permute(iseed,n,nmax,iordre)
C
C 1. Test [a] = [a+b] -----
      if(notest1.eq.1) goto 15
C Permute raw data Y ***
C      if(mode.eq.1) then
         do 12 i=1,n
         do 12 j=1,p
   12    Yper(i,j)=Y(iordre(i),j)
         SSYper=SSY
C Permute Yres2, the residuals of the reduced regression model Y|B
C         else
C         do 14 i=1,n
C         do 14 j=1,p
C   14    Yper(i,j)=Yres2(iordre(i),j)
CC         call SofSq(n,p,nmax,pmax,Yper,SSYper)
C         endif
C Test without covariables
      call Produit(Resing1,Yper,Yfitper, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      call SofSq(n,p,nmax,pmax,Yfitper,ABper)
      call Produit(Resing2,Yper,Yfitper, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      call SofSq(n,p,nmax,pmax,Yfitper,BCper)
      F1per=ABper*dfloat(idf2_1)/(BCper*dfloat(idf1_1))
      if(F1per.ge.F1) iGEF1=iGEF1+1
C
C 2. Test b = [b+c] -----
   15 if(notest2.eq.1) goto 90
C Permute raw data Y
C      if(mode.eq.1) then
C         if(notest1.eq.1) then
C            do 16 i=1,n
C            do 16 j=1,p
C   16       Yper(i,j)=Y(iordre(i),j)
C            SSYper=SSY
C            endif
C Permute Yres1, the residuals of the reduced regression model Y|A ***
C         else
         do 18 i=1,n
         do 18 j=1,p
   18    Yper(i,j)=Yres1(iordre(i),j)
         call SofSq(n,p,nmax,pmax,Yper,SSYper)
C         endif
C Test without covariables
C      if((notest1.eq.1).or.(mode.eq.2)) then
         call Produit(Resing2,Yper,Yfitper, n,n,p, nmax,nmax, 
     +                nmax,pmax, nmax,pmax)
         call SofSq(n,p,nmax,pmax,Yfitper,BCper)
C         endif
      call Produit(Resing3,Yper,Yfitper, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      call SofSq(n,p,nmax,pmax,Yfitper,ABCper)
      Dper=SSYper-ABCper
      F2per=BCper*dfloat(idf2_2)/(Dper*dfloat(idf1_2))
      if(F2per.ge.F2) iGEF2=iGEF2+1
C
   90 continue
C End of permutation test ===================================
C
C Write results to output file
   80 continue
C Results for fraction A
      if(notest1.eq.0) then
         if(mode.eq.0) then
            write(4,102) idf1_1,idf2_1,AB,AB/dfloat(idf1_1),F1
            else
            Pperm=dfloat(iGEF1)/dfloat(nperm+1)
            sig=' '
            if(Pperm.le.0.05) sig='*'
            write(4,102) idf1_1,idf2_1,AB,AB/dfloat(idf1_1),F1,Pperm,sig
            endif
         else
         write(4,102) idf1_1,idf2_1,AB,dfloat(idf1_1)
         endif
C Results for fraction C
      if(notest2.eq.0) then
         if(mode.eq.0) then
            write(4,103) idf1_2,idf2_2,BC,BC/dfloat(idf1_2),F2
            else
            Pperm=dfloat(iGEF2)/dfloat(nperm+1)
            sig=' '
            if(Pperm.le.0.05) sig='*'
            write(4,103) idf1_2,idf2_2,BC,BC/dfloat(idf1_2),F2,Pperm,sig
            endif
         else
         write(4,103) idf1_2,idf2_2,BC,BC/dfloat(idf1_2)
         endif
C
      write(4,104) idf2_2,D,D/dfloat(idf2_2)
      write(4,120)
      write(4,105) (n-1),SSY
      write(4,120)
C
      fin=secnds(debut)
      write(*,111) fin
      write(4,111) fin
C
C Compute the fitted canonical scores
      if(job.eq.1) return
      debut=secnds(0.0)
C 1. [a] = [a+b]
      write(LUT,*) 'Object scores fitted to the main factor'
      call PCA(n,nmax,p,pmax,nombre,Yfit1,Trans,ntrmax,U,Z,vect,Lambda,
     +         EE,SSY,LUT,1,1,nvalpr)
      write(LUT,*) '                          -------------------------'
      write(LUT,*)
C 2. [b] = [b+c]
      write(LUT,*) 'Object scores fitted to the nested factor'
      call PCA(n,nmax,p,pmax,nombre,Yfit2,Trans,ntrmax,U,Z,vect,Lambda,
     +         EE,SSY,LUT,1,1,nvalpr)
      write(LUT,*) '                          -------------------------'
      write(LUT,*)
C 6. [d]
      write(LUT,*) 
     +   'Object scores, PCA of unexplained variation (residuals)'
      call PCA(n,nmax,p,pmax,nombre,Yres3,Trans,ntrmax,U,Z,vect,Lambda,
     +         EE,SSY,LUT,0,1,nvalpr)
      write(LUT,*) '                          -------------------------'
C
      fin=secnds(debut)
      write(*,112) fin
      write(LUT,112) fin
C
      return
C              123456789012345678901234512345123456789012345123456789012
  101 format( ' Source of variation  df1  df2               SS        ',
     +        '   MS           F      P(perm)')
  102 format( ' Main factor        ',2i5,f20.5,f13.5,f12.5,f10.5,1x,a1)
  103 format(/' Nested factor      ',2i5,f20.5,f13.5,f12.5,f10.5,1x,a1)
  202 format( ' Main factor        ',2i5,f20.5,f13.5,4x,
     +        'Cannot be tested')
  203 format(/' Nested factor      ',2i5,f20.5,f13.5,4x,
     +        'Cannot be tested')
  104 format(/' Within subgroups   ',i5,5x,f20.5,f13.5)
  105 format( ' Total              ',i5,5x,f20.5)
  110 format(/' Permutation test: please wait...'/)
  111 format(/' Computation time for nested anova:',f10.2,' sec.'/)
  112 format(/' Computation time for canonical object scores:',
     +       f10.2,' sec.'/)
  120 format(' -------------------------------------------------------',
     +        '--------------------------------')
      end

      Subroutine Part3(n,nmax,p,pmax,m1,m2,maxfileX,mtot,mmax,ntrmax,
     +   Y,X,Xwk,job,mode,nperm,Trans,Inv,Work,SSY,
     +   Yper,Yfitper,Z,U,vect,Yfitwk,Yfit1,Yfit2,Yfit3,Yfit7,
     +   Yres1,Yres2,Yres3,Yres4,Yres5,Yres6,Yres7,Resing1,Resing2,
     +   Resing3,Resing4,Resing5,Resing6,Resing7,iGauss1,iGauss2,
     +   iGauss3,GaussB,Lambda,EE,iordre,nombre,LUT,debut)
      Integer n,nmax,p,pmax,mmax,ntrmax
      Integer m1(maxfileX),m2(maxfileX),iordre(nmax),nombre(ntrmax)
      Integer iGauss1(mmax),iGauss2(mmax),iGauss3(mmax)
      Real*8 GaussB(mmax,1)
      Real*8 Y(nmax,pmax),Yfitwk(nmax,pmax),
     +       Yfit1(nmax,pmax),Yfit2(nmax,pmax),Yfit3(nmax,pmax),
     +       Yfit7(nmax,pmax),
     +       Yres1(nmax,pmax),Yres2(nmax,pmax),Yres3(nmax,pmax),
     +       Yres4(nmax,pmax),Yres5(nmax,pmax),Yres6(nmax,pmax),
     +       Yres7(nmax,pmax),Yper(nmax,pmax),Yfitper(nmax,pmax)
      Real*8 X(nmax,mmax),Xwk(nmax,mmax)
      Real*8 Trans(ntrmax,nmax),Inv(mmax,mmax),Work(nmax,mmax)
      Real*8 Resing1(nmax,nmax),Resing2(nmax,nmax),Resing3(nmax,nmax),
     +       Resing4(nmax,nmax),Resing5(nmax,nmax),Resing6(nmax,nmax),
     +       Resing7(nmax,nmax)
      Real*8 Z(nmax,ntrmax),U(ntrmax,ntrmax),vect(pmax),
     +       Lambda(ntrmax),EE(ntrmax)
      Real*8 SSY,SSYper,Pperm,temp
      Real*8 ADFG,BDEG,CEFG,ABCDEFG,H,DEFG
      Real*8 ADFGper,BDEGper,CEFGper,ABCDEFGper,Hper
      Real*8 F1,F2,F3,F1per,F2per,F3per
      Character sig*1,fraction*9
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
      write(4,*)
      write(4,120)
      write(4,101)
      write(4,120)
C Part3: There are 3 factors: main A, nested B(A), nested C(B(A))
C
C 1. Compute [a] = [a+d+f+g] (Y.X1) -----
C Degrees of freedom for the F1 and F1per statistics
      notest1=0
      idf1_1=m2(1)
      idf2_1=m2(2)-m2(1)
      mwk=0
      call SelectX(n,nmax,m1,m2,maxfileX,1,X,Xwk,mmax,mwk)
C Compute residualizing matrix  Resing1(nxn) = X inv[X'X] X'
      call Residize(n,mwk, nmax,mmax, ntrmax,nmax, mmax,
     +       nmax,mmax, nmax, Xwk,Trans,Inv,Work,Resing1,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      call Produit(Resing1,Y,Yfit1, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      do i=1,n
         do j=1,p
            Yres1(i,j)=Y(i,j)-Yfit1(i,j)
            enddo
         enddo
C Yres1 contains the residuals of Y after fitting factor A
C Fractions: ADFG = fitted, BCEH = residuals
      call SofSq(n,p,nmax,pmax,Yfit1,ADFG)
C
C 2. Compute [b] = [b+d+e+g] (Y.X2) -----
C Degrees of freedom for the F2 and F2per statistics
      notest2=0
      idf1_2=m2(2)-m2(1)
      idf2_2=m2(3)-m2(2)
      mwk=0
      call SelectX(n,nmax,m1,m2,maxfileX,2,X,Xwk,mmax,mwk)
C Compute residualizing matrix  Resing2(nxn) = X inv[X'X] X'
      call Residize(n,mwk, nmax,mmax, ntrmax,nmax, mmax,
     +       nmax,mmax, nmax, Xwk,Trans,Inv,Work,Resing2,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      call Produit(Resing2,Y,Yfit2, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      do i=1,n
         do j=1,p
            Yres2(i,j)=Y(i,j)-Yfit2(i,j)
            enddo
         enddo
C Yres2 contains the residuals of Y after fitting factor B
C Fractions: BDEG = fitted, ACFH = residuals
      call SofSq(n,p,nmax,pmax,Yfit2,BDEG)
C
C 3. Compute [c] = [c+e+f+g] (Y.X3) -----
C Degrees of freedom for the F3 and F3per statistics
      notest3=0
      idf1_3=m2(3)-m2(2)
      idf2_3=n-1-m2(3)
      mwk=0
      call SelectX(n,nmax,m1,m2,maxfileX,3,X,Xwk,mmax,mwk)
C Compute residualizing matrix  Resing3(nxn) = X inv[X'X] X'
      call Residize(n,mwk, nmax,mmax, ntrmax,nmax, mmax,
     +       nmax,mmax, nmax, Xwk,Trans,Inv,Work,Resing3,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      call Produit(Resing3,Y,Yfit3, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      do i=1,n
         do j=1,p
            Yres3(i,j)=Y(i,j)-Yfit3(i,j)
            enddo
         enddo
C Yres3 contains the residuals of Y after fitting factor C
C Fractions: CEFG = fitted, ABDH = residuals
      call SofSq(n,p,nmax,pmax,Yfit3,CEFG)
C
C 4. Compute (Y.X2,X3) or Y|(B,C) and residuals -----
      mwk=0
      call SelectX(n,nmax,m1,m2,maxfileX,2,X,Xwk,mmax,mwk)
      call SelectX(n,nmax,m1,m2,maxfileX,3,X,Xwk,mmax,mwk)
C Compute residualizing matrix  Resing4(nxn) = X inv[X'X] X'
      call Residize(n,mwk, nmax,mmax, ntrmax,nmax, mmax,
     +       nmax,mmax, nmax, Xwk,Trans,Inv,Work,Resing4,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      call Produit(Resing4,Y,Yres4, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      do i=1,n
         do j=1,p
            Yres4(i,j)=Y(i,j)-Yres4(i,j)
            enddo
         enddo
C Yres4 contains the residuals of Y after fitting B and C
C
C 5. Compute (Y.X1,X3) or Y|(A,C) and residuals -----
      mwk=0
      call SelectX(n,nmax,m1,m2,maxfileX,1,X,Xwk,mmax,mwk)
      call SelectX(n,nmax,m1,m2,maxfileX,3,X,Xwk,mmax,mwk)
C Compute residualizing matrix  Resing5(nxn) = X inv[X'X] X'
      call Residize(n,mwk, nmax,mmax, ntrmax,nmax, mmax,
     +       nmax,mmax, nmax, Xwk,Trans,Inv,Work,Resing5,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      call Produit(Resing5,Y,Yres5, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      do i=1,n
         do j=1,p
            Yres5(i,j)=Y(i,j)-Yres5(i,j)
            enddo
         enddo
C Yres5 contains the residuals of Y after fitting A and C
C
C 6. Compute (Y.X1,X2) or Y|(A,B) and residuals -----
      mwk=0
      call SelectX(n,nmax,m1,m2,maxfileX,1,X,Xwk,mmax,mwk)
      call SelectX(n,nmax,m1,m2,maxfileX,2,X,Xwk,mmax,mwk)
C Compute residualizing matrix  Resing6(nxn) = X inv[X'X] X'
      call Residize(n,mwk, nmax,mmax, ntrmax,nmax, mmax,
     +       nmax,mmax, nmax, Xwk,Trans,Inv,Work,Resing6,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      call Produit(Resing6,Y,Yres6, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      do i=1,n
         do j=1,p
            Yres6(i,j)=Y(i,j)-Yres6(i,j)
            enddo
         enddo
C Yres6 contains the residuals of Y after fitting A and B
C
C 7. Compute [a+b+c+d+e+f+g] (Y.X1,X2,X3) and residuals [h] -----
C Degrees of freedom for the F7 and F7per statistics
      notest7=0
      m=m2(3)
      idf1_7=m
      idf2_7=n-1-m
      mwk=0
      call SelectX(n,nmax,m1,m2,maxfileX,1,X,Xwk,mmax,mwk)
      call SelectX(n,nmax,m1,m2,maxfileX,2,X,Xwk,mmax,mwk)
      call SelectX(n,nmax,m1,m2,maxfileX,3,X,Xwk,mmax,mwk)
C Compute residualizing matrix  Resing3(nxn) = X inv[X'X] X'
      call Residize(n,mwk, nmax,mmax, ntrmax,nmax, mmax,
     +       nmax,mmax, nmax, Xwk,Trans,Inv,Work,Resing7,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      call Produit(Resing7,Y,Yfit7, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      do i=1,n
         do j=1,p
            Yres7(i,j)=Y(i,j)-Yfit7(i,j)
            enddo
         enddo
C Fractions: ABCDEFG = fitted, H = residuals
      call SofSq(n,p,nmax,pmax,Yfit7,ABCDEFG)
      H=SSY-ABCDEFG
C
C Test statistics for nested anova
      if((idf1_1.le.0).or.(idf2_1.le.0).or.(BDEG.le.0.0)) notest1=1
      if((idf1_2.le.0).or.(idf2_2.le.0).or.(CEFG.le.0.0)) notest2=1
      if((idf1_3.le.0).or.(idf2_3.le.0).or.(H.le.0.0)) notest3=1
      F1=ADFG*dfloat(idf2_1)/(BDEG*dfloat(idf1_1))
      F2=BDEG*dfloat(idf2_2)/(CEFG*dfloat(idf1_2))
      F3=CEFG*dfloat(idf2_3)/(H   *dfloat(idf1_3))
C Compute [d+e+f+g] to check the orthogonality of the coded factors
      DEFG=ADFG+BDEG+CEFG-ABCDEFG
      if(DEFG.gt.0.000000001) then
         write(*,*) 'Error in coding the nested factors: [d+e+f+g] =',B
         Stop
         endif
C
C Prepare for the permutation tests
      if(nperm.eq.0) goto 92
      do i=1,n
         iordre(i)=i
         enddo
      iseed=0
      do i=1,100
         turn=rand(iseed)
         enddo
      iGEF1=1
      iGEF2=1
      iGEF3=1
C Begin permutation test =====================================
      write(*,110)
      nval=n*p
      nlimit=300
      do 90 iperm=1,nperm
      if((nval.gt.nlimit).and.(mod(iperm,10).eq.0)) write(*,130) iperm
  130 format('+','Permutation no',i6)
      call Permute(iseed,n,nmax,iordre)
C
C 1. Test [a] = [a+d+f+g] -----
      if(notest1.eq.1) goto 15
C Permute raw data Y ***
C      if(mode.eq.1) then
         do 12 i=1,n
         do 12 j=1,p
   12    Yper(i,j)=Y(iordre(i),j)
         SSYper=SSY
C Permute Yres4, the residuals of the reduced model Y|(B,C)
C         else
C         do 14 i=1,n
C         do 14 j=1,p
C   14    Yper(i,j)=Yres4(iordre(i),j)
C         call SofSq(n,p,nmax,pmax,Yper,SSYper)
C         endif
C Test without covariables
      call Produit(Resing1,Yper,Yfitper, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      call SofSq(n,p,nmax,pmax,Yfitper,ADFGper)
C
      call Produit(Resing2,Yper,Yfitper, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      call SofSq(n,p,nmax,pmax,Yfitper,BDEGper)
      F1per=ADFGper*dfloat(idf2_1)/(BDEGper*dfloat(idf1_1))
      if(F1per.ge.F1) iGEF1=iGEF1+1
C
C 2. Test [b] = [b+d+e+g] -----
   15 if(notest2.eq.1) goto 20
C Permute raw data Y
C      if(mode.eq.1) then
C         if(notest1.eq.1) then
C            do 16 i=1,n
C            do 16 j=1,p
C   16       Yper(i,j)=Y(iordre(i),j)
C            SSYper=SSY
C            endif
C Permute Yres5, the residuals of the reduced model Y|(A,C) ***
C         else
         do 18 i=1,n
         do 18 j=1,p
   18    Yper(i,j)=Yres5(iordre(i),j)
C         call SofSq(n,p,nmax,pmax,Yper,SSYper)
C         endif
C Test without covariables
C      if((notest1.eq.1).or.(mode.eq.2)) then
         call Produit(Resing2,Yper,Yfitper, n,n,p, nmax,nmax, 
     +                nmax,pmax, nmax,pmax)
         call SofSq(n,p,nmax,pmax,Yfitper,BDEGper)
C         endif
C
      call Produit(Resing3,Yper,Yfitper, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      call SofSq(n,p,nmax,pmax,Yfitper,CEFGper)
      F2per=BDEGper*dfloat(idf2_2)/(CEFGper*dfloat(idf1_2))
      if(F2per.ge.F2) iGEF2=iGEF2+1
C
C 3. Test [c] = [c+e+f+g] -----
   20 if(notest3.eq.1) goto 90
C Permute raw data Y
C      if(mode.eq.1) then
C         if((notest1*notest2).eq.1) then
C            do 22 i=1,n
C            do 22 j=1,p
C   22       Yper(i,j)=Y(iordre(i),j)
C            SSYper=SSY
C            endif
C Permute Yres6, the residuals of the reduced model Y|(A,B) ***
C         else
         do 24 i=1,n
         do 24 j=1,p
   24    Yper(i,j)=Yres6(iordre(i),j)
         call SofSq(n,p,nmax,pmax,Yper,SSYper)
C         endif
C Test without covariables
C      if((notest2.eq.1).or.(mode.eq.2)) then
         call Produit(Resing3,Yper,Yfitper, n,n,p, nmax,nmax, 
     +                nmax,pmax, nmax,pmax)
         call SofSq(n,p,nmax,pmax,Yfitper,CEFGper)
C         endif
C
      call Produit(Resing7,Yper,Yfitper, n,n,p, nmax,nmax, nmax,pmax, 
     +             nmax,pmax)
      call SofSq(n,p,nmax,pmax,Yfitper,ABCDEFGper)
      Hper=SSYper-ABCDEFGper

      F3per=CEFGper*dfloat(idf2_3)/(Hper*dfloat(idf1_3))
C     F3   =CEFG   *dfloat(idf2_3)/(H   *dfloat(idf1_3))
      if(F3per.ge.F3) iGEF3=iGEF3+1
C
   90 continue
C End of permutation test ===================================
C
C Write results to output file
   92 continue
C Results for fraction A
      if(notest1.eq.0) then
         if(mode.eq.0) then
            write(4,102) idf1_1,idf2_1,ADFG,ADFG/dfloat(idf1_1),F1
            else
            Pperm=dfloat(iGEF1)/dfloat(nperm+1)
            sig=' '
            if(Pperm.le.0.05) sig='*'
        write(4,102) idf1_1,idf2_1,ADFG,ADFG/dfloat(idf1_1),F1,Pperm,sig
            endif
         else
         write(4,202) idf1_1,idf2_1,ADFG,ADFG/dfloat(idf1_1)
         endif
C Results for fraction B
      if(notest2.eq.0) then
         if(mode.eq.0) then
            write(4,103) idf1_2,idf2_2,BDEG,BDEG/dfloat(idf1_2),F2
            else
            Pperm=dfloat(iGEF2)/dfloat(nperm+1)
            sig=' '
            if(Pperm.le.0.05) sig='*'
        write(4,103) idf1_2,idf2_2,BDEG,BDEG/dfloat(idf1_2),F2,Pperm,sig
            endif
         else
         write(4,203) idf1_2,idf2_2,BDEG,BDEG/dfloat(idf1_2)
         endif
C Results for fraction C
      if(notest3.eq.0) then
         if(mode.eq.0) then
            write(4,104) idf1_3,idf2_3,CEFG,CEFG/dfloat(idf1_3),F3
            else
            Pperm=dfloat(iGEF3)/dfloat(nperm+1)
            sig=' '
            if(Pperm.le.0.05) sig='*'
        write(4,104) idf1_3,idf2_3,CEFG,CEFG/dfloat(idf1_3),F3,Pperm,sig
            endif
         else
         write(4,204) idf1_3,idf2_3,CEFG,CEFG/dfloat(idf1_3)
         endif
C
      write(4,105) idf2_3,H,H/dfloat(idf2_3)
      write(4,120)
      write(4,106) (n-1),SSY
      write(4,120)
C
      fin=secnds(debut)
      write(*,111) fin
      write(4,111) fin
C
C Compute the fitted canonical scores
      if(job.eq.1) return
      debut=secnds(0.0)
C 1. [a] = [a+d+f+g]
      write(LUT,*) 'Object scores fitted to the main factor'
      call PCA(n,nmax,p,pmax,nombre,Yfit1,Trans,ntrmax,U,Z,vect,Lambda,
     +         EE,SSY,LUT,1,1,nvalpr)
      write(LUT,*) '                          -------------------------'
      write(LUT,*)
C 2. [b] = [b+d+e+g]
      write(LUT,*) 'Object scores fitted to the first nested factor'
      call PCA(n,nmax,p,pmax,nombre,Yfit2,Trans,ntrmax,U,Z,vect,Lambda,
     +         EE,SSY,LUT,1,1,nvalpr)
      write(LUT,*) '                          -------------------------'
      write(LUT,*)
C 3. [c] = [c+e+f+g]
      write(LUT,*) 'Object scores fitted to the second nested factor'
      call PCA(n,nmax,p,pmax,nombre,Yfit3,Trans,ntrmax,U,Z,vect,Lambda,
     +         EE,SSY,LUT,1,1,nvalpr)
      write(LUT,*) '                          -------------------------'
      write(LUT,*)
C 21. [h]
      write(LUT,*) 
     +   'Object scores, PCA of unexplained variation (residuals)'
      call PCA(n,nmax,p,pmax,nombre,Yres7,Trans,ntrmax,U,Z,vect,Lambda,
     +         EE,SSY,LUT,0,1,nvalpr)
      write(LUT,*) '                          -------------------------'
C
      fin=secnds(debut)
      write(*,112) fin
      write(LUT,112) fin
C
      return
C              123456789012345678901234512345123456789012345123456789012
  101 format( ' Source of variation  df1  df2               SS        ',
     +        '   MS           F      P(perm)')
  102 format( ' Main factor        ',2i5,f20.5,f13.5,f12.5,f10.5,1x,a1)
  103 format(/' 1st nested factor  ',2i5,f20.5,f13.5,f12.5,f10.5,1x,a1)
  104 format(/' 2nd nested factor  ',2i5,f20.5,f13.5,f12.5,f10.5,1x,a1)
  202 format( ' Main factor        ',2i5,f20.5,f13.5,4x,
     +        'Cannot be tested')
  203 format(/' 1st nested factor  ',2i5,f20.5,f13.5,4x,
     +        'Cannot be tested')
  204 format(/' 2nd nested factor  ',2i5,f20.5,f13.5,4x,
     +        'Cannot be tested')
  105 format(/' Within subgroups   ',i5,5x,f20.5,f13.5)
  106 format( ' Total              ',i5,5x,f20.5)
  110 format(/' Permutation test: please wait...'/)
  111 format(/' Computation time for nested anova:',f10.2,' sec.'/)
  112 format(/' Computation time for canonical object scores:',
     +       f10.2,' sec.'/)
  120 format(' -------------------------------------------------------',
     +        '--------------------------------')
      end

      Subroutine PCA(n,nmax,p,pmax,nombre,Yhat,Trans,ntrmax,U,Z,vect,
     +               Lambda,EE,SSY,LUT,ican,iprint,nvalpr)
      Integer n,nmax,p,pmax,mmax,ntrmax,nombre(ntrmax)
      Real*8 Yhat(nmax,pmax),Z(nmax,ntrmax),U(ntrmax,ntrmax)
      Real*8 Lambda(ntrmax),EE(ntrmax)
      Real*8 Trans(ntrmax,nmax),vect(pmax),temp,dflnm1,SSY,totinert
C In canonical analysis, nvalpr = min[(n-1),p,m], 
C but in the analysis of residuals, nvalpr = min[(n-1),p].
C Hence, nombre, Z, U, Lambda, and EE must be dimensioned to ntrmax.
C
C Matrix Yhat contains the fitted values of the regressions
      irdacca=1
      langue=2
      dflnm1=dfloat(n-1)
      nvalpr=min(n-1,p)
      totinert=SSY/dflnm1
C Compute SSCP matrix (Yhat'Yhat), followed by Householder decomposition
      do 2 i=1,n
      do 2 j=1,p
    2 Trans(j,i)=Yhat(i,j)
      call Produit(Trans,Yhat,U, p,n,p, ntrmax,nmax, nmax,pmax, 
     +             ntrmax,ntrmax)
      call tred2(U,p,ntrmax,Lambda,EE)
      call tqli(Lambda,EE,p,ntrmax,U)
C      write(LUT,100) (Lambda(j), j=1,p)
C Put the eigenvalues and corresponding eigenvectors in descending order.
C Phase 1: Trier les valeurs propres ainsi que le vecteur d'entiers "nombre"
C Phase 1: Sort eigenvalues together with the vector of integers "nombre"
         do 30 i=1,p
   30    nombre(i)=i
         do 36 j=2,p
            temp=Lambda(j)
            nn=nombre(j)
            do 32 i=j-1,1,-1
               if(Lambda(i).gt.temp) goto 34
               Lambda(i+1)=Lambda(i)
               nombre(i+1)=nombre(i)
   32          continue
            i=0
   34       Lambda(i+1)=temp
            nombre(i+1)=nn
   36    continue
C      write(LUT,100) (Lambda(j), j=1,p)
C Phase 2: Sort U according to new order of "nombre". Use "vect" as buffer.
         do 42 jj=1,p
         do 38 j=1,p
   38    vect(j)=U(jj,j)
         do 40 j=1,p
   40    U(jj,j)=vect(nombre(j))
   42    continue
C Phase 3: eliminate null eigenvalues
      nval=0
      do 44 i=1,p
      if(Lambda(i).gt.0.000001) then
         nval=nval+1
         else
         goto 46
         endif
   44 continue
   46 if(nval.lt.nvalpr) nvalpr=nval
      if(nval.eq.0) then
         temp=0.0
         write(LUT,*)
      write(LUT,*) 'Total variance is 0. There are no ordination axes.'
         write(LUT,*)
         return
         endif
C      write(LUT,100) (Lambda(j), j=1,p)
C 
      if(iprint.eq.0) return
C Print eigenvalues
      if(nvalpr.eq.0) then
         write(LUT,*) 
     +      ' Total variance is 0. There are no canonical axes.'
         return
         endif
      if(ican.eq.1) then
         if(langue.eq.1) write(LUT,101) nvalpr
         if(langue.eq.2) write(LUT,201) nvalpr
         else
         if(langue.eq.1) write(LUT,102) nvalpr
         if(langue.eq.2) write(LUT,202) nvalpr
         endif
      if(irdacca.eq.1) then
         do 48 j=1,nvalpr
   48    Lambda(j)=Lambda(j)/dflnm1
         endif
      write(LUT,100) (Lambda(i), i=1,nvalpr)
      if(irdacca.eq.1) then
         if(langue.eq.1) write(LUT,108) 
         if(langue.eq.2) write(LUT,208)
         else
         if(langue.eq.1) write(LUT,103) 
         if(langue.eq.2) write(LUT,203)
         endif
      write(LUT,100) (100.0*Lambda(i)/totinert, i=1,nvalpr)
      temp=0.0
      do 52 i=1,nvalpr
      temp=temp+Lambda(i)
   52 vect(i)=temp
      if(irdacca.eq.1) then
         if(langue.eq.1) write(LUT,109) 
         if(langue.eq.2) write(LUT,209)
         else
         if(langue.eq.1) write(LUT,104)
         if(langue.eq.2) write(LUT,204)
         endif
      write(LUT,100) (100.0*vect(i)/totinert, i=1,nvalpr)
      temp=0.0
      do 54 i=1,nvalpr
   54 temp=temp+Lambda(i)
      if(ican.eq.1) then
         if(langue.eq.1) write(LUT,105)
         if(langue.eq.2) write(LUT,205)
         else
         if(langue.eq.1) write(LUT,111)
         if(langue.eq.2) write(LUT,211)
         endif
         write(LUT,100) (100.0*vect(i)/temp, i=1,nvalpr)
   56 continue
      if(ican.eq.1) then
         if(langue.eq.1) write(LUT,106) temp
         if(langue.eq.2) write(LUT,206) temp
         else
         if(langue.eq.1) write(LUT,107) temp
         if(langue.eq.2) write(LUT,207) temp
         endif
      if(langue.eq.1) write(LUT,112)
      if(langue.eq.2) write(LUT,212)
C Cadrage = 1: U et Z / Scaling = 1: U and Z
      call Produit(Yhat,U,Z, n,p,nvalpr, nmax,pmax, ntrmax,ntrmax, 
     +             nmax,ntrmax) 
      do 64 i=1,n
   64 write(LUT,100) (Z(i,j), j=1,nvalpr)
      write(LUT,*)
C     Calculer la longueur des vecteurs de Z / Compute length of the vectors of Z
      do 66 j=1,nvalpr
   66 vect(j)=0.0
      do 68 i=1,n
      do 68 j=1,nvalpr
   68 vect(j)=vect(j)+Z(i,j)**2
      write(LUT,*) 'Variances'
      write(LUT,100) (vect(j)/dfloat(n-1), j=1,nvalpr)
      write(LUT,*)
C
      return
  100 format(1x,200F10.5)
  101 format(/' Valeurs propres canoniques (',i4,' valeurs propres)')
  201 format(/' Canonical eigenvalues (',i4,' eigenvalues)')
  102 format
     +   (/' Valeurs propres non canoniques (',i4,' valeurs propres)')
  202 format(/' Non-canonical eigenvalues (',i4,' eigenvalues)')
  103 format(/' % de la variance totale de l''AFC')
  203 format(/' % of total variance of CA')
  104 format(/' % cumulatif de la variance totale de l''AFC')
  204 format(/' Cumulative % of total variance of CA')
  105 format(/' % cumulatif de la variance canonique')
  205 format(/' Cumulative % of canonical variance')
  106 format(/' Somme des valeurs propres canoniques',F10.5)
  206 format(/' Sum of all canonical eigenvalues',F10.5)
  107 format(/' Somme des valeurs propres non canoniques',F10.5)
  207 format(/' Sum of non-canonical eigenvalues',F10.5)
  108 format(/' % de la variance totale de l''ACP')
  208 format(/' % of total variance of PCA')
  109 format(/' % cumulatif de la variance totale de l''ACP')
  209 format(/' Cumulative % of total variance of PCA')
  111 format(/' % cumulatif de la variance residuelle')
  211 format(/' Cumulative % of residual variance')
  112 format(/' Ordination des objets')
  212 format(/' Object scores')
      end
            
      Subroutine Permute (iseed,n,nmax,iordre)
C This subroutine permutes the first 'n' values of vector 'iordre' at random
C in an equiprobable way. This property has been checked through intensive 
C simulations.
      Integer iseed,n,i,itemp,km1,m
      Integer iordre(nmax)
      m=n
      km1=n-1
      do 10 i=1,km1
    8    j = 1 + rand(iseed)*m
         if(j.gt.m) goto 8
         itemp = iordre(m)
         iordre(m) = iordre(j)
         iordre(j) = itemp
         m=m-1
   10 continue
      return
      end

      Subroutine Produit(Z1,Z2,Z3,n1,n2,n3,n1imax,n1jmax,n2imax,n2jmax,
     +   n3imax,n3jmax)
      Real*8 Z1(n1imax,n1jmax),Z2(n2imax,n2jmax),Z3(n3imax,n3jmax),temp
C Z1 * Z2 = Z3
      do 8 i=1,n1
      do 8 j=1,n3
      temp=0.0
      do 6 k=1,n2
    6 temp=temp+Z1(i,k)*Z2(k,j)
    8 Z3(i,j)=temp
      return
      end
            
      Subroutine Read_Y(n,p,nmax,pmax,Y,colsum,ntran,SSY,idebug)
      Integer n,nmax,p,pmax
      Real*8 Y(nmax,pmax),colsum(pmax),SSY,temp
      Character*79 nomfich
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
      read(2,*) nomfich
      open(11,file=nomfich,status='OLD')
C Read the response data file
      read(11,*) n,p
      write(*,102) nomfich,n,p
      write(4,102) nomfich,n,p
      if((n.gt.nmax).or.(p.gt.pmax)) then
         write(*,103)
         stop
         endif
      do i=1,n
         read(11,*) (Y(i,j), j=1,p)
         enddo
      close(11)
C Transform Y
      if(ntran.eq.0) then
         write(4,*) 'No transformation of the species data'
         else
         call Transform(n,nmax,p,pmax,Y,colsum,ntran)
         endif
C Centre Y
      call Centre(n,nmax,1,p,pmax,Y)
      if(idebug.eq.1) then
         write(4,*) ' First 10 columns of matrix Y'
         write(4,*)
         do i=1,n
            write(4,100) (Y(i,j), j=1,10)
            enddo
         endif
C     Compute the total SS in Y: SSY = trace(Y'Y)
      SSY=0.0
      do i=1,n
         do j=1,p
            temp=Y(i,j)
            SSY=SSY+temp*temp
            enddo
         enddo
      write(4,104) SSY
C
      return
  100 format(10f10.5)
  102 format(/' File of response variables:       ',a32/
     +        i6,' objects'/
     +        i6,' species')
  103 format(' File too large. Redimension the program.')
  104 format(' Total sum of squares in Y [transformed] (SSY) =',f19.5/)
      end

      Subroutine Read_X(n,nmax,mmax,ntrmax,LIN,X,Xwk,Trans,Inv,m1,m2,
     +       nfiles,maxfileX,mtot,no,iflag1,iflag2,icheck,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      Integer m1(maxfileX),m2(maxfileX)
      Integer iGauss1(mmax),iGauss2(mmax),iGauss3(mmax)
      Real*8 GaussB(mmax,1)
      Real*8 X(nmax,mmax),Xwk(nmax,mmax),Trans(ntrmax,nmax),
     +       Inv(mmax,mmax),temp
      Character*79 nomfich
C Reads the various files of explanatory variables into matrix X.
C m1(4) and m2(4) give the beginning and end of each group of explanatory var.
      read(2,*) nomfich
      open(LIN,file=nomfich,status='OLD')
      read(LIN,*) nobj,nvar
      m1(no)=mtot+1
      m2(no)=mtot+nvar
      mtot=m2(no)
      write(*,102) (LIN-11),nomfich,nobj,nvar
      write(4,102) (LIN-11),nomfich,nobj,nvar
      if(mtot.gt.mmax) then
         write(*,103)
         write(4,103)
         stop
         endif
      if(n.ne.nobj) stop 'Mismatch between Y and explanatory file'
C
      ibegin=m1(no)
      iend  =m2(no)
      do i=1,nobj
         read(LIN,*) (X(i,j), j=ibegin,iend)
         enddo
      write(4,*)
      close(LIN)
C Check that each dummy variable sums to 0
      if(icheck.eq.1) then
         iflag1=0
         do j=ibegin,iend
            temp=0.0
            do i=1,nobj
               temp=temp+X(i,j)
               enddo
            if(temp.ne.0.0) then
               iflag1=1
               write(*,104) no
               write(4,104) no
               endif
            enddo
C Check orthogonality within file of explanatory variables
         do j=ibegin,iend-1
            do jj=j+1,iend
               temp=0.0
               do i=1,nobj
                  temp=temp+X(i,j)*X(i,jj)
                  enddo
               if(temp.ne.0.0) then
                  iflag1=1
                  write(*,105) no
                  write(4,105) no
                  endif
               enddo
            enddo
         endif
C      write(*,*) 'Point 1: iflag1 =', iflag1
C Check collinearity within the file of explanatory variables
      do i=1,nobj
         do j=1,nvar
            jj=j+ibegin-1
            Xwk(i,j)=X(i,jj)
            enddo
         enddo
      call Centre(n,nmax,1,nvar,mmax,Xwk)
      do i=1,nobj
         do j=1,nvar
            Trans(j,i)=Xwk(i,j)
            enddo
         enddo
      call Produit(Trans,Xwk,Inv, nvar,nobj,nvar, ntrmax,nmax, 
     +             nmax,mmax, mmax,mmax)
      call GaussJ(nvar,mmax,Inv,iGauss1,iGauss2,iGauss3,GaussB,iflag2)
      if(iflag2.eq.1) write(*,106) no
      if(iflag2.eq.1) write(4,106) no
C
      return
  100 format(8f10.5)
  102 format(' File containing factor ',i1,':         ',a32/
     +        i6,' objects'/
     +        i6,' dummy variables')
  103 format(
     +' *** Too many explanatory variables. Redimension the program.'/)
  104 format(
     +' *** The variables coding for factor',i2,' do not all sum to 0'/)
  105 format(
     +' *** The variables coding for factor',i2,' are not orthogonal'/)
  106 format(
     +' *** The variables coding for factor',i2,' are collinear'/)
      end

      Subroutine Residize(n,m, nmax,mmax, trimax,trjmax, invmax,
     +       wkimax,wkjmax, rsmax,X,Trans,Inv,Work,Resing,
     +       iGauss1,iGauss2,iGauss3,GaussB)
      Integer n,m,nmax,mmax,trimax,trjmax,invmax,wkimax,wkjmax,rsmax
      Integer iGauss1(mmax),iGauss2(mmax),iGauss3(mmax)
      Real*8 GaussB(mmax,1)
      Real*8 X(nmax,mmax),Trans(trimax,trjmax),Inv(invmax,invmax),
     +       Work(wkimax,wkjmax),Resing(rsmax,rsmax)
      Real*8 temp
C This subroutine computes the residualizing matrix  Resing = X Inv[X'X] X'
C Following that, a regression is simply computed as Yfit = Resing*Y
      if(m.eq.1) then
         temp=0.0
         do i=1,n
            temp=temp+X(i,1)*X(i,1)
            enddo
         temp=1.0/temp
         do i=1,n
            Work(i,1)=X(i,1)*temp
            enddo
         do j=1,n
            do i=1,n
               Resing(i,j)=Work(i,1)*X(j,1)
               enddo
            enddo         
        else
         do i=1,n
            do j=1,m
               Trans(j,i)=X(i,j)
               enddo
            enddo
         call Produit(Trans,X,Inv, m,n,m, 
     +                trimax,trjmax, nmax,mmax, invmax,invmax)
         call GaussJ(m,mmax,Inv,iGauss1,iGauss2,iGauss3,GaussB,iflag)
         if(iflag.eq.1) Stop 'Singular matrix'
         call Produit(X,Inv,Work, n,m,m, 
     +                nmax,mmax, invmax,invmax, wkimax,wkjmax)
         call Produit(Work,Trans,Resing, n,m,n, 
     +                wkimax,wkjmax, trimax,trjmax, rsmax,rsmax)
        endif
      return
      end

      Subroutine SelectX(n,nmax,m1,m2,maxfileX,nomat,X,Xwk,mmax,mwk)
C Writes selected explanatory variables to Xwk
C 'mwk' is the total number of variables in file Xwk
      Integer n,m1(maxfileX),m2(maxfileX),mwk,nmax,mmax
      Real*8 X(nmax,mmax),Xwk(nmax,mmax)
C
      jstart=m1(nomat)
      jend  =m2(nomat)
C
      do 10 j=jstart,jend
      mwk=mwk+1
      do 10 i=1,n
   10 Xwk(i,mwk)=X(i,j)
C     
C      do 20 i=1,n
C   20 write(*,100) (Xwk(i,j), j=1,mwk)
C  100 format(8f10.5)
      return
      end
      
      Subroutine SofSq(n,p,nmax,pmax,Mat,SSY)
      Integer n,p,nmax,pmax
      Real*8 Mat(nmax,pmax),sx,sx2,dfln,SSY,temp
C Computes the sum of squared deviations from the means (SSY) in a matrix
      dfln=dfloat(n)
      SSY=0.0
      do 20 j=1,p
      sx=0.0
      sx2=0.0
      do 10 i=1,n
      temp=Mat(i,j)
      sx=sx+temp
   10 sx2=sx2+temp*temp
      temp=sx2-(sx*sx/dfln)
   20 SSY=SSY+temp
      return
      end

      Subroutine Transform(n,nmax,p,pmax,mat,colsum,ntran)
      Integer n,nmax,p,pmax
      Real*8 mat(nmax,pmax),colsum(pmax),rowsum,tsum
C Computing the transformed variables.
C The code is written in such a way as to avoid computing in advance
C a vector 'rowsum(i)' of length 'n'
      iflag1=0
      iflag2=0
      if((ntran.eq.2).or.(ntran.eq.3)) then
         tsum=0.0
         do 14 j=1,p
         colsum(j)=0.0
         do 14 i=1,n
         tsum=tsum+mat(i,j)
   14    colsum(j)=colsum(j)+mat(i,j)
         do 16 j=1,p
         if(colsum(j).eq.0.0) then
            write(*,102) j
            iflag1=1
            endif
   16    continue
         endif
C Check for negative species frequencies
      do i=1,n
         do j=1,p
            if(Mat(i,j).lt.0.0) write(*,103) i,j
            enddo
         enddo
C
      goto (20,30,40,50,60) ntran
C Chord distance
   20 write(4,*) 'Chord transformation'
      do 26 i=1,n
      rowsum=0.0
      do 22 j=1,p
   22 rowsum=rowsum+mat(i,j)**2
      if(rowsum.eq.0.0) then
         write(*,100) i
         iflag2=2
         goto 26
         endif
      rowsum=dsqrt(rowsum)
      do 24 j=1,p
   24 mat(i,j)=mat(i,j)/rowsum
   26 continue
      goto 68
C Chi-square metric
   30 write(4,*) 'Chi-square metric transformation'
      do 36 i=1,n
      rowsum=0.0
      do 32 j=1,p
   32 rowsum=rowsum+mat(i,j)
      if(rowsum.eq.0.0) then
         write(*,100) i
         iflag2=2
         goto 36
         endif
      do 34 j=1,p
      if(colsum(j).ne.0.0) 
     +  mat(i,j)=mat(i,j)/(rowsum*dsqrt(colsum(j)))
   34 continue
   36 continue
      goto 68
C Chi-square distance
   40 write(4,*) 'Chi-square distance transformation'
      do 46 i=1,n
      rowsum=0.0
      do 42 j=1,p
   42 rowsum=rowsum+mat(i,j)
      if(rowsum.eq.0.0) then
         write(*,100) i
         iflag2=2
         goto 46
         endif
      do 44 j=1,p
      if(colsum(j).ne.0.0) 
     +  mat(i,j)=dsqrt(tsum)*mat(i,j)/(rowsum*dsqrt(colsum(j)))
   44 continue
   46 continue
      goto 68
C Distance between species profiles
   50 write(4,*) 'Species profile transformation'
      do 56 i=1,n
      rowsum=0.0
      do 52 j=1,p
   52 rowsum=rowsum+mat(i,j)
      if(rowsum.eq.0.0) then
         write(*,100) i
         iflag2=2
         goto 56
         endif
      do 54 j=1,p
   54 mat(i,j)=mat(i,j)/rowsum
   56 continue
      goto 68
C Hellinger distance
   60 write(4,*) 'Hellinger transformation'
      do 66 i=1,n
      rowsum=0.0
      do 62 j=1,p
   62 rowsum=rowsum+mat(i,j)
      if(rowsum.eq.0.0) then
         write(*,100) i
         iflag2=2
         goto 66
         endif
      do 64 j=1,p
   64 mat(i,j)=dsqrt(mat(i,j)/rowsum)
   66 continue
C
   68 continue
      if(iflag1.eq.1) stop
     +   'Column sums of 0 are incompatible with this transformation.'
C      write(2,*) 'Data after transformation'
C      do 70 i=1,n
C   70 write(2,101) (mat(i,j),j=1,p)
      return
  100 format(' Transformation subroutine warning: Row',i4,
     +       ' has total species abundance = 0')
  101 format(8f10.5)
  102 format(' Transformation subroutine: Column',i4,
     +       ' has total species abundance = 0')
  103 format(' Negative species frequency in row',i4,' column',i4)
      end

C Subroutine GaussJ is modified from Numerical Recipes.
C
C The author of the present program is not authorized to distribute the source
C code of Numerical Recipes subroutines. If you want to use this subroutine to
C recompile the present program (in the original or some modified form) or in
C any other program, you should obtain a license from Numerical Recipes. 
C See the WWWeb site <http://world.std.com/~nr/storefront.html>.
C The subroutine is reproduced here to show how the original Numerical Recipes
C subroutine should be modified before incorporating it in the present program.
C
      Subroutine GaussJ(m,mmax,A,ipiv,indxr,indxc,B,iflag)
      Real*8 A(mmax,mmax),B(mmax,1),val,big,pivinv,dum
      Integer*4 ipiv(mmax), indxr(mmax), indxc(mmax)
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
c-----------------------------------------------------------------------
c 
c Linear equation solution by Gauss-Jordan elimination.
c From 'Numerical Recipes' (Press et al., 1986), p. 28-29.
c A is an input matrix of 'm x m' elements, stored in a array of physical
c dimensions 'mmax x mmax'. B is an input matrix of 'm x ncolB' containing the
c 'ncolB' right-hand side vectors, stored in an array of physical dimension 
c 'mmax x 1'.
c [In this implementation, B only contains a column vector of zero on start.]
c On output, A is replaced by its matrix inverse, and B is replaced by
c the corresponding set of solution vectors.
c 
c Note: a singular matrix causes an appropriate "pause" message. 
c 
c The integer arrays IPIV, INDXR and INDXC are used for bookkeeping on the
c pivoting. 'mmax' should be as large as the largest anticipated value of 'm'.
c 
c Modified for GSLIB from Numerical Recipes (1986), Section 2.1, p. 28-29.
c 
C Solves the system: [B(nxm)|identity] = A(nxn).[X(nxm)|invA] where ncolB=1.
c X (stored in B on input) is the initial solution vector of the set of
c linear equations; on output, B is the set of regression coefficients.
c
c Pour realiser une regression lineaire:
c au depart, B est le vecteur des valeurs X'y;
c A est la matrice [X'X] a inverser;
C a la fin du programme, B contient le vecteur des coefficients de regression.
c-----------------------------------------------------------------------
      if (m .lt. 1) stop 'illogical m in Gaussj.'
      ncolB=1
      iflag=0
C     Initialisation de b
      do 2 i=1,m
      do 2 j=1,ncolB
    2 b(i,j)=0.0
C
      do 10 j = 1, m
      ipiv(j) = 0
   10 continue 
C Beginning the main loop (110) over the columns to be reduced
      do 110 i = 1, m
        big = 0.
C Beginning the outer loop (30) of the search for a pivot element
        do 30 j = 1, m
          if (ipiv(j) .ne. 1) then 
            do 20 k = 1, m
              if (ipiv(k) .eq. 0) then 
                if (abs(a(j,k)) .ge. big) then 
                  big = abs(a(j,k))
                  irow = j
                  icol = k
                end if 
              else if (ipiv(k) .gt. 1) then
                pause 'singular matrix'
              end if 
   20       continue 
          end if 
   30   continue 
        ipiv(icol) = ipiv(icol) + 1
C We now have the pivot element, so we interchange rows, if needed, to put
C the pivot element on the diagonal. The columns are not physically
C interchanged, only relabeled: INDX(i), the column of the i-th pivot, is
C the i-th column that is reduced, while INDXR(i) is the row in which that
C pivot element was originally located. If INDXR(i)<>INDXC(i) there is an
C implied column interchange. With this form of bookkeeping, the solution 
C B's will end up in the correct order, and the inverse matrix will be
C scrambled by columns.
        if (irow .ne. icol) then 
          do 40 l = 1, m
            dum = a(irow,l)
            a(irow,l) = a(icol,l)
            a(icol,l) = dum
   40     continue 
          do 50 l = 1, ncolB
            dum = b(irow,l)
            b(irow,l) = b(icol,l)
            b(icol,l) = dum
   50     continue 
        end if 
        indxr(i) = irow
        indxc(i) = icol
C We are now ready to divide the pivot row by the pivot element, 
C located at Irow and Icol
C        if (dabs(a(icol,icol)) .le. 0.00000001) pause 'Singular matrix.'
        if (dabs(a(icol,icol)) .le. 0.00000001) then
           iflag=1
C           write(*,*) 'Singular matrix'
           return
           endif
        pivinv = 1. / a(icol,icol)
        a(icol,icol) = 1.
        do 60 l = 1, m
          a(icol,l) = a(icol,l) * pivinv
   60   continue 
        do 70 l = 1, ncolB
          b(icol,l) = b(icol,l) * pivinv
   70   continue 
C Next we reduce the rows, except for the pivot one, of course
        do 100 ll = 1, m
          if (ll .ne. icol) then 
            dum = a(ll,icol)
            a(ll,icol) = 0.
            do 80 l = 1, m
              a(ll,l) = a(ll,l) - a(icol,l) * dum
   80       continue 
            do 90 l = 1, ncolB
              b(ll,l) = b(ll,l) - b(icol,l) * dum
   90       continue 
          end if 
  100   continue 
  110 continue 
C End of the main loop over columns of the reduction. 
C It only remains to unscramble the solution in view of the column
C interchanges. We do this by interchanging pairs of columns in the
C reverse order that the permutation was built up.
      do 130 l = m, 1, -1
        if (indxr(l) .ne. indxc(l)) then 
          do 120 k = 1, m
            dum = a(k,indxr(l))
            a(k,indxr(l)) = a(k,indxc(l))
            a(k,indxc(l)) = dum
  120     continue 
        end if 
  130 continue 
C
C End of inversion procedure.
      return
      end

C Following are two subroutines for finding the eigenvalues of a real
C symmetric matrix using the Householder method, a procedure recommended
C by Numerical Recipes (Press et al., 1986): routines TRED2 and TQLI.

      Subroutine TRED2(A,N,NP,D,E)
C
C Householder reduction of a real, symmetric, N by N matrix A, stored in
C an NP by NP physical array.  On output, A is replaced by the orthogonal
C matrix Q effecting the transformation.  D returns the diagonal elements
C of the tridiagonal matrix, and E the off-diagonal elements, with E(1) = 0.
C Several statements, as noted in comments, can be omitted if only
C eigenvalues are to be found, in which case A contains no useful information
C on output.  Otherwise they are to be included.
C
      Dimension A(NP,NP), D(NP), E(NP)
C REAL*8 statement added for double precision.
      Real*8 A,D,E,H,SCALE,F,G,HH
      
      IF (N.GT.1) THEN
         DO 18 I = N,2,-1
              L = I - 1
              H = 0.
              SCALE = 0.
              IF (L.GT.1) THEN
                 DO 11 K = 1,L
                    SCALE = SCALE + DABS(A(I,K))
11                  CONTINUE
              IF (SCALE.EQ.0) THEN
                 E(I) = A(I,L)
              ELSE
                 DO 12 K = 1,L
                    A(I,K) = A(I,K)/SCALE
                    H = H + A(I,K)**2
12               CONTINUE
                 F = A(I,L)
                 G = -SIGN(DSQRT(H),F)
                 E(I) = SCALE*G
                 H = H-F*G
                 A(I,L) = F-G
                 F = 0.
                 DO 15 J = 1,L
C Omit following line if finding only eigenvalues.
                    A(J,I) = A(I,J)/H
                    G = 0.
                    DO 13 K = 1,J
                       G = G + A(J,K) * A(I,K)
13                  CONTINUE
                    IF (L.GT.J) THEN
                       DO 14 K = J+1, L
                          G = G + A(K,J)*A(I,K)
14                     CONTINUE
                    END IF
                    E(J) = G/H
                    F = F + E(J)*A(I,J)
15                  CONTINUE
                 HH = F/(H + H)
                 DO 17 J = 1,L
                    F = A(I,J)
                    G = E(J) - HH*F
                    E(J) = G
                    DO 16 K = 1,J
                       A(J,K) = A(J,K) - F*E(K) - G*A(I,K)
16                  CONTINUE
17               CONTINUE
              END IF
              ELSE
                 E(I) = A(I,L)
              END IF
              D(I) = H
18       CONTINUE
      END IF
C Omit following line if finding only eigenvalues.
      D(1) = 0.
      E(1) = 0.
      DO 23 I = 1,N
C Delete lines from here...
         L = I-1
         IF (D(I).NE.0.) THEN
            DO 21 J = 1,L
               G = 0.
               DO 19 K = 1,L
                  G = G + A(I,K) * A(K,J)
19             CONTINUE
               DO 20 K = 1,L
                  A(K,J) = A(K,J) - G*A(K,I)
20             CONTINUE
21          CONTINUE
         END IF
C ... to here when finding only eigenvalues.
         D(I) = A(I,I)
C Also delete lines from here ....
         A(I,I) = 1.
         IF(L.GE.1) THEN
            DO 22 J = 1,L
               A(I,J) = 0.
               A(J,I) = 0.
22          CONTINUE
         END IF
C ... to here when finding only eigenvalues.
23    continue
      return
      end

      Subroutine TQLI (D,E,N,NP,Z)
C
C QL algorithm with implicit shifts, to determine the eigenvalues and
C eigenvectors of a real, symmetric, tridiagonal matrix previously
C reduced by TRED 2 (Numerical Recipes).  D is a vector of length NP.
C On input, its first N elements are the diagonal elements of the tridiagonal
C matrix.  On output, it returns the eigenvalues.  The vector E inputs the
C subdiagonal elements of the tridiagonal matrix, with E(1) arbitrary.  On
C output, E is destroyed.  When finding only the eigenvalues, several lines
C may be omitted, as noted in the coments.  If the eigenvectors of the
C tridiagonal matrix are desired, the matrix Z (N by N matrix stored in
C NP by NP array) is input as the identity matrix.  If the eigenvectors of
C a matrix that has been reduced by TRED2 are required, then Z is input as
C the matrix output by TRED2.  In either case, the Kth column of Z returns
C the normalized eigenvector corresponding to D(K).
C
      Dimension D(NP), E(NP), Z(NP,NP)
C REAL*8 statement added to original code for double precision.
      Real*8 D,E,Z,DD,G,R,S,C,P,F,B

      IF (N.GT.1) THEN
         DO 11 I = 2,N
            E(I-1) = E(I)
11       CONTINUE
         E(N) = 0.
         DO 15 L = 1,N
            ITER = 0
1           DO 12 M = L, N-1
               DD = DABS(D(M)) + DABS(D(M+1))
               IF (DABS(E(M)) + DD.EQ.DD) GO TO 2
12          CONTINUE
            M = N
2           IF (M.NE.L) THEN
               IF (ITER.EQ.50) Stop 'Too many iterations in TQLI'
               ITER = ITER + 1
               G = (D(L+1)-D(L))/(2.*E(L))
               R = SQRT(G**2+1.)
               G = D(M) - D(L) + E(L)/(G + SIGN(R,G))
               S = 1.
               C = 1.
               P = 0.
               DO 14 I = M-1,L,-1
                  F = S*E(I)
                  B = C*E(I)
                  IF (DABS(F).GE.DABS(G)) THEN
                     C = G/F
                     R = DSQRT(C**2+1.)
                     E(I+1) = F*R
                     S = 1./R
                     C = C*S
                  ELSE
                     S = F/G
                     R = DSQRT(S**2+1.)
                     E(I+1) = G*R
                     C = 1./R
                     S = S*C
                  END IF
                  G = D(I+1) - P
                  R = (D(I) - G)*S+2.*C*B
                  P = S*R
                  D(I+1) = G + P
                  G = C*R - B
C Omit lines from here ...
                  DO 13 K = 1,N
                     F = Z(K,I+1)
                     Z(K,I+1) = S*Z(K,I) + C*F
                     Z(K,I) = C*Z(K,I) - S*F
13                CONTINUE
C ... to here when finding only eigenvalues.
14             CONTINUE
               D(L) = D(L) - P
               E(L) = G
               E(M) = 0.
               GO TO 1
            END IF
15          CONTINUE
      end if
      return
      end
